!Copyright 2016 J.-M. CHESNEAUX, F. JEZEQUEL, J.-L. LAMOTTE
!
!This file is part of CADNA.
!
!    CADNA is free software: you can redistribute it and/or modify it
!    under the terms of the GNU Lesser General Public License as
!    published by the Free Software Foundation, either version 3 of the
!    License, or (at your option) any later version.
!
!    CADNA is distributed in the hope that it will be useful, but WITHOUT
!    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
!    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
!    Public License for more details.
!
!    You should have received a copy of the GNU Lesser General Public
!    License along with CADNA. If not, see
!    <http://www.gnu.org/licenses/>.
!
!****h module cadna_lt
!  NAME
!    cadna_lt
!
!
!  DESCRIPTION
!    Defines all the functions involving at least one argument
!    of stochastic type which overload the "<" operator
!    in a test such as "a<b".
!
!  INPUTS
!    All the functions have two arguments of stochastic, 
!    real or integer type in single or double precision. At least 
!    one argument must be of stochastic type.
!
!  RETURN VALUE
!    All the functions have one logical return value. 
!
!  NOTES
!    The return value takes into account the stochastic definition of
!    the "<" operator (see the documentation).
!
!******
!
module cadna_lt
  !
  use cadna_type
  use cadna_cestac
  !
  interface operator (<)
     !	=======================
     module procedure lt_st_st, &
          &		lt_single_st, lt_st_single, lt_int_st, lt_st_int, &
          &		lt_dst_dst, lt_dst_double, lt_double_dst, &
          &		lt_dst_st, lt_st_dst, lt_dst_single, lt_single_dst, &
          &		lt_st_double, lt_double_st, lt_int_dst, lt_dst_int, &
          &             lt_vvdst_double
  end interface
  !
!!!!!!!!!!!!!!!!!!!!!
contains   !!
!!!!!!!!!!!!!!!!!!!!!
  !
  !
  logical function lt_st_st(a,b)
    type  (single_st), intent(in) :: a, b
    logical :: r
    type  (single_st) :: aux
    aux%x = a%x - b%x
    aux%y = a%y - b%y
    aux%z = a%z - b%z
    r = computed_zero_st(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_st_st = (.not.r.and.( (a%x+a%y+a%z).lt.(b%x+b%y+b%z) ))
  end function lt_st_st
  !
  logical function lt_single_st(a,b)
    type  (single_st), intent(in) :: b
    real, intent(in) :: a
    logical r
    type  (single_st) :: aux
    aux%x = a - b%x
    aux%y = a - b%y
    aux%z = a - b%z
    r = computed_zero_st(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_single_st = (.not.r.and.( (a+a+a).lt.(b%x+b%y+b%z) ))
  end function lt_single_st
  !
  logical function lt_st_single(a,b)
    type  (single_st), intent(in) :: a
    real, intent(in) :: b
    logical r
    type  (single_st) :: aux
    aux%x = a%x - b
    aux%y = a%y - b
    aux%z = a%z - b
    r = computed_zero_st(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_st_single = (.not.r.and.( (a%x+a%y+a%z).lt.(b+b+b) ))
  end function lt_st_single
  !
  logical function lt_int_st(a,b)
    type  (single_st), intent(in) :: b
    integer, intent(in) :: a
    logical r
    type  (single_st) :: aux
    aux%x = a - b%x
    aux%y = a - b%y
    aux%z = a - b%z
    r = computed_zero_st(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_int_st = (.not.r.and.( (a+a+a).lt.(b%x+b%y+b%z) ))
  end function lt_int_st
  !
  logical function lt_st_int(a,b)
    type  (single_st), intent(in) :: a
    integer, intent(in) :: b
    logical r
    type  (single_st) :: aux
    aux%x = a%x - b
    aux%y = a%y - b
    aux%z = a%z - b
    r = computed_zero_st(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_st_int = (.not.r.and.( (a%x+a%y+a%z).lt.(b+b+b) ))
  end function lt_st_int
  !
  logical function lt_dst_dst(a,b)
    type (double_st), intent(in) :: a,b
    logical :: r
    type  (double_st) :: aux
    aux%x = a%x - b%x
    aux%y = a%y - b%y
    aux%z = a%z - b%z
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_dst_dst = (.not.r.and.( (a%x+a%y+a%z).lt.(b%x+b%y+b%z) ))
  end function lt_dst_dst
  !
  logical function lt_dst_double(a,b)
    type (double_st), intent(in) :: a
    double precision, intent(in) :: b
    logical r
    type  (double_st) :: aux
    aux%x = a%x - b
    aux%y = a%y - b
    aux%z = a%z - b
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_dst_double = (.not.r.and.( (a%x+a%y+a%z).lt.(b+b+b) ))
  end function lt_dst_double
  !
  logical function lt_double_dst(a,b)
    double precision, intent(in) :: a
    type (double_st), intent(in) :: b
    logical r
    type  (double_st) :: aux
    aux%x = a - b%x
    aux%y = a - b%y
    aux%z = a - b%z
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_double_dst = (.not.r.and.( (a+a+a).lt.(b%x+b%y+b%z) ))
  end function lt_double_dst
  !
  logical function lt_dst_st(a,b)
    type (double_st), intent(in) :: a
    type (single_st), intent(in) :: b
    logical r
    type  (double_st) :: aux
    aux%x = a%x - b%x
    aux%y = a%y - b%y
    aux%z = a%z - b%z
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_dst_st = (.not.r.and.( (a%x+a%y+a%z).lt.(b%x+b%y+b%z) ))
  end function lt_dst_st
  !
  logical function lt_st_dst(a,b)
    type (single_st), intent(in) :: a
    type (double_st), intent(in) :: b
    logical r
    type  (double_st) :: aux
    aux%x = a%x - b%x
    aux%y = a%y - b%y
    aux%z = a%z - b%z
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_st_dst = (.not.r.and.( (a%x+a%y+a%z).lt.(b%x+b%y+b%z) ))
  end function lt_st_dst
  !
  logical function lt_single_dst(a,b)
    real, intent(in) :: a
    type (double_st), intent(in) :: b
    logical r
    type  (double_st) :: aux
    aux%x = a - b%x
    aux%y = a - b%y
    aux%z = a - b%z
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_single_dst = (.not.r.and.( (a+a+a).lt.(b%x+b%y+b%z) ))
  end function lt_single_dst
  !
  logical function lt_dst_single(a,b)
    type (double_st), intent(in) :: a
    real, intent(in) :: b
    logical r
    type  (double_st) :: aux
    aux%x = a%x - b
    aux%y = a%y - b
    aux%z = a%z - b
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_dst_single = (.not.r.and.( (a%x+a%y+a%z).lt.(b+b+b) ))
  end function lt_dst_single
  !
  logical function lt_st_double(a,b)
    type (single_st), intent(in) :: a
    double precision, intent(in) :: b
    logical r
    type  (double_st) :: aux
    aux%x = a%x - b
    aux%y = a%y - b
    aux%z = a%z - b
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_st_double = (.not.r.and.( (a%x+a%y+a%z).lt.(b+b+b) ))
  end function lt_st_double
  !
  logical function lt_double_st(a,b)
    double precision, intent(in) :: a
    type (single_st), intent(in) :: b
    logical r
    type  (double_st) :: aux
    aux%x = a - b%x
    aux%y = a - b%y
    aux%z = a - b%z
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_double_st = (.not.r.and.( (a+a+a).lt.(b%x+b%y+b%z) ))
  end function lt_double_st
  !
  logical function lt_int_dst(a,b)
    type  (double_st), intent(in) :: b
    integer, intent(in) :: a
    logical r
    type  (double_st) :: aux
    aux%x = a - b%x
    aux%y = a - b%y
    aux%z = a - b%z
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
   lt_int_dst = (.not.r.and.( (a+a+a).lt.(b%x+b%y+b%z) ))
  end function lt_int_dst
  !
  logical function lt_dst_int(a,b)
    type  (double_st), intent(in) :: a
    integer, intent(in) :: b
    logical r
    type  (double_st) :: aux
    aux%x = a%x - b
    aux%y = a%y - b
    aux%z = a%z - b
    r = computed_zero_dst(aux)
    if (cadna_branching_tag) then
       if (r) then
          call instability(cadna_branching)
       endif
    endif
    lt_dst_int = (.not.r.and.( (a%x+a%y+a%z).lt.(b+b+b) ))
  end function lt_dst_int
  !
  function lt_vvdst_double(a,b)
    type (double_st), dimension(:,:), intent(in) :: a
    double precision, intent(in) :: b
    logical, dimension(size(a,1),size(a,2)) :: lt_vvdst_double
    integer :: i,j
    do j=1,size(a,2)
       do i=1,size(a,1)
          lt_vvdst_double(i,j)=lt_dst_double(a(i,j),b)
       enddo
    enddo
  end function lt_vvdst_double
  !
end module cadna_lt
