!Copyright 2016 J.-M. CHESNEAUX, F. JEZEQUEL, J.-L. LAMOTTE
!
!This file is part of CADNA.
!
!    CADNA is free software: you can redistribute it and/or modify it
!    under the terms of the GNU Lesser General Public License as
!    published by the Free Software Foundation, either version 3 of the
!    License, or (at your option) any later version.
!
!    CADNA is distributed in the hope that it will be useful, but WITHOUT
!    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
!    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
!    Public License for more details.
!
!    You should have received a copy of the GNU Lesser General Public
!    License along with CADNA. If not, see
!    <http://www.gnu.org/licenses/>.
!
!****h module cadna_str
!  NAME
!    cadna_str
!
!
!  DESCRIPTION
!    Contains the functions for printing out stochastic variables.
!
!******
!
module cadna_str
  !
  use cadna_type
  use cadna_cestac
  !
!****f cadna_str/str
!  NAME
!    str
!
!  FUNCTION
!    Functions of the interface "str" convert a stochastic value 
!    to a string containing the scientific notation of this argument.
!    Only the exact significant digits appear in the string.
!
!  INPUTS
!    a : single_st (or double_st)
!
!  RETURN VALUE
!    A string of 14 characters if the argument is a "single_st" variable
!    and of 23 characters if the argument is a "double_st" variable.
!
!******
  interface str
     !       ======================
     module procedure st_to_char, dst_to_char
  end interface
  !
!!!!!!!!!!!!!!!!!!!!!
contains   !!
!!!!!!!!!!!!!!!!!!!!!
  !
  !
  character(len=14) function st_to_char(a)
    implicit none
    type (single_st), intent(in):: a
    real res,fract_res
    integer prec_res,nn,nres,naux,i
    real :: bux=1.0
    logical :: b
    b = (bux+1.0e-20.eq.1.0) 
    call rnd_arr
    st_to_char(1:14) = '              '
!-----------------------12345678901234-
    prec_res = nb_significant_digit_st(a)
    if (prec_res.eq.0) then
       st_to_char(1:3) = achar(64)//'.0'
    else
       prec_res = min(prec_res,7)
       res = (a%x + a%y + a%z) / 3.0
       if (res.lt.0.0) then
          st_to_char(1:3) = '-0.'
       else
          st_to_char(1:3) = ' 0.'
       end if
       res = abs(res)
       if (res.eq.0.0) then
          nn = 0
          fract_res = 0.0
       else
          if (res.ge.1.0) then
             nn = int(log10(res)) + 1
          else
             nn = int(log10(res))
          end if
          fract_res = res*10.0**(-nn)
          if (fract_res.lt.0.1) then
             fract_res = 0.1
          endif
          if (fract_res.ge.1.0) then
             fract_res = 0.1
             nn = nn+1
          endif
       end if
       naux = prec_res+3
       do i=4,naux-1
          nres = int(fract_res*10.0)
          st_to_char(i:i) = achar(48+ nres)
          fract_res = 10.0*fract_res - nres
       enddo
       nres = min(9,nint(fract_res*10.0))
       st_to_char(i:i) = achar(48+ nres)
       i = i+1
       st_to_char(i:i) = 'E'
       i = i+1
       if (nn.lt.0) then
          st_to_char(i:i) = '-'
       else
          st_to_char(i:i) = '+'
       end if
       nn = abs(nn)
       i = i+1
       st_to_char(i:i) = achar(48 + nn/10)
       i = i+1
       st_to_char(i:i) = achar(48 + mod(nn,10))
    end if
    if (b) then
       call rnd_moinf
    else
       call rnd_plinf
    endif
  end function st_to_char
  !
  character(len=23) function dst_to_char(a)
    implicit none
    type (double_st), intent(in):: a
    double precision res,fract_res
    integer prec_res,nres,nn,naux,i
    real :: bux=1.0
    logical :: b
    b = (bux+1.0e-20.eq.1.0)
    call rnd_arr
    dst_to_char(1:23) = '                       '
!------------------------12345678901234567890123-
    prec_res = nb_significant_digit_dst(a)
    if (prec_res.eq.0) then
       dst_to_char(1:3) = achar(64)//'.0'
    else
       prec_res = min(prec_res,15)
       res = (a%x + a%y + a%z) / 3.0
       if (res.lt.0.0) then
          dst_to_char(1:3) = '-0.'
       else
          dst_to_char(1:3) = ' 0.'
       end if
       res = abs(res)
       if (res.eq.0.0) then
          nn = 0
          fract_res = 0.d0
       else
          if (res.ge.1.0) then
             nn = int(log10(res)) + 1
          else
             nn = int(log10(res))
          end if
          fract_res = res*10.d0**(-nn)
          if (fract_res.lt.0.1d0) then
             fract_res = 0.1d0
          endif
          if (fract_res.ge.1.d0) then
             fract_res = 0.1d0
             nn = nn+1
          endif
       end if
       naux = prec_res+3
       do i=4,naux-1
          nres = int(fract_res*10.d0)
          dst_to_char(i:i) = achar(48 + nres)
          fract_res = 10.d0*fract_res - nres
       enddo
       nres = min(9,nint(fract_res*10.d0))
       dst_to_char(i:i) = achar(48 + nres)
       i=i+1
       dst_to_char(i:i) = 'E'
       i = i+1
       if (nn.lt.0) then
          dst_to_char(i:i) = '-'
       else
          dst_to_char(i:i) = '+'
       end if
       nn = abs(nn)
       i = i+1
       dst_to_char(i:i) = achar(48 + nn/100)
       nn = mod(nn,100)
       i = i+1
       dst_to_char(i:i) = achar(48 + nn/10)
       i = i+1
       dst_to_char(i:i) = achar(48 + mod(nn,10))
    end if
    if (b) then
       call rnd_moinf
    else
       call rnd_plinf
    endif
  end function dst_to_char
!
end module cadna_str
